// edtFTPnet
// 
// Copyright (C) 2004 Enterprise Distributed Technologies Ltd
// 
// www.enterprisedt.com
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Bug fixes, suggestions and comments should posted on 
// http://www.enterprisedt.com/forums/index.php
// 
// Change Log:
// 
// $Log: OS400FileParser.cs,v $
// Revision 1.3  2007-05-23 00:29:36  hans
// Added TimeIncludesSeconds property.
//
// Revision 1.2  2007-02-13 12:12:16  bruceb
// extra comment
//
// Revision 1.1  2007/02/02 01:41:26  bruceb
// new parser
//
//

using System;
using System.Globalization;
using System.Text;
using EnterpriseDT.Util;
using EnterpriseDT.Util.Debug;

namespace EnterpriseDT.Net.Ftp
{    
    /// <summary>  
    /// Represents a remote OS400 file parser. 
    /// </summary>
    /// <remarks>
    /// This parser is somewhat experimental :)
    /// </remarks>
    /// <author>Bruce Blackshaw
    /// </author>
    /// <version>$Revision: 1.3 $</version>
    public class OS400FileParser : FTPFileParser
    {            
        /// <summary>Directory field</summary>
        private static readonly string DIR = "*DIR";
    
        /// <summary>Directory field</summary>
        private static readonly string FILE1 = "*FILE";
        
        /// <summary>Directory field</summary>
        private static readonly string FILE2 = "*STMF";
    
        /// <summary>MEM field?</summary>
        private static readonly string MEM = "*MEM";
    
        /// <summary> Number of expected fields</summary>
        private static readonly int MIN_EXPECTED_FIELD_COUNT = 6;

        /// <summary>Date formats</summary>
        private static readonly string DATE_FORMAT_1 = "dd'/'MM'/'yy' 'HH':'mm':'ss";	
        private static readonly string DATE_FORMAT_2 = "dd'/'MM'/'yyyy' 'HH':'mm':'ss";	
        private static readonly string DATE_FORMAT_3 = "dd'.'MM'.'yy' 'HH':'mm':'ss";	

        private static readonly string DATE_FORMAT_11 = "yy'/'MM'/'dd' 'HH':'mm':'ss";	
        private static readonly string DATE_FORMAT_12 = "yyyy'/'MM'/'dd' 'HH':'mm':'ss";	
        private static readonly string DATE_FORMAT_13 = "yy'.'MM'.'dd' 'HH':'mm':'ss";	

        /// <summary>array of formats</summary>
        private static string[] formats1 = {DATE_FORMAT_1,DATE_FORMAT_2,DATE_FORMAT_3};
        private static string[] formats2 = {DATE_FORMAT_11,DATE_FORMAT_12,DATE_FORMAT_13};
        private string[] formats = formats1;

        /// <summary> Logging object</summary>
        private Logger log = Logger.GetLogger("OS400FileParser");

        /// <summary>
        /// Get flag indicating whether or not the most recent parse returned seconds.
        /// </summary>
        public override bool TimeIncludesSeconds
        {
            get { return true; }
        }
        
        /// <summary> Parse server supplied string</summary>
        /// <param name="raw">raw string to parse</param>
        /// <returns>FTPFile object representing the raw string</returns>
        /// <remarks>Listing look like the below:
        ///        CFT             45056 04/12/06 14:19:31 *FILE AFTFRE1.FILE
        ///        CFT                                     *MEM AFTFRE1.FILE/AFTFRE1.MBR
        ///        CFT             36864 28/11/06 15:19:30 *FILE AFTFRE2.FILE
        ///        CFT                                     *MEM AFTFRE2.FILE/AFTFRE2.MBR
        ///        CFT             45056 04/12/06 14:19:37 *FILE AFTFRE6.FILE
        ///        CFT                                     *MEM  AFTFRE6.FILE/AFTFRE6.MBR
        ///        QSYSOPR         28672 01/12/06 20:08:04 *FILE FPKI45POK5.FILE
        ///        QSYSOPR                                 *MEM FPKI45POK5.FILE/FPKI45POK5.MBR        
        /// </remarks>
        public override FTPFile Parse(string raw) 
        {
            string[] fields = StringSplitter.Split(raw);      
            
            // skip blank lines
            if(fields.Length <= 0)
                return null;
            // skip line which lists MEM
            if (fields.Length >= 2 && fields[1].Equals(MEM))
                return null;
            if (fields.Length < MIN_EXPECTED_FIELD_COUNT)
                return null;
            
            // first field is name
            string owner = fields[0];

            // next is size
            long size = Int64.Parse(fields[1]);

            string lastModifiedStr = fields[2] + " " + fields[3];
            DateTime lastModified = GetLastModified(lastModifiedStr);
 
            // test is dir
            bool isDir = false;
            if (fields[4] == DIR)
                isDir = true; 

            string name = fields[5];
            
            FTPFile file = new FTPFile(FTPFile.OS400, raw, name, size, isDir, ref lastModified); 
            file.Owner = owner;
            return file;
        }

        private DateTime GetLastModified(string lastModifiedStr) 
        {
            DateTime lastModified;
            try 
            {
                lastModified = DateTime.ParseExact(lastModifiedStr, formats, 
                    ParsingCulture.DateTimeFormat, DateTimeStyles.None);

                // if incorrect date, try second bunch of formats with year at beginning
                if (lastModified > DateTime.Now) 
                {
                    formats = formats2;
                    lastModified = DateTime.ParseExact(lastModifiedStr, formats2, 
                        ParsingCulture.DateTimeFormat, DateTimeStyles.None);
                    log.Debug("Swapping to year first formats (found date in future)");
                    throw new RestartParsingException();
                }
            }
            catch (FormatException ex1)
            {
                try 
                {
                    formats = formats2;
                    lastModified = DateTime.ParseExact(lastModifiedStr, formats, 
                        ParsingCulture.DateTimeFormat, DateTimeStyles.None);
                    log.Debug("Swapping to year first formats (" + ex1.Message + ")");
                    throw new RestartParsingException();
                }
                catch (FormatException ex2)
                {
                    lastModified = DateTime.Parse(lastModifiedStr,ParsingCulture.DateTimeFormat);
                }
            }
            return lastModified;
        }

        
    }
}
